
const { VArray, Variable, Span, Div, P } = require('alkali');
const { onEvent, getElementSize, getElementPosition, styleAttributes } = require('./tools');
const Knob = require('./Knob');

/*
Options:
 - knobs (number of knobs)
 - max
 - min
 - valueSeparator
 - attached
 - ready
*/


/*
	TODO:
	make axis choose-able
*/

class Slider {
	constructor(parent, opts = {}, changeCallback){

		if (typeof parent == 'string') parent = document.getElementById(parent);

		this.axisLength = new Variable(0);
		this.axisHeight = new Variable(0);
		this.sliderAxis = new Div('.sliderAxis');
		this.options = opts;
		const visibility = new Variable(0);
		// create reactive variables for knob internal positioning
		const knobPositions = [...Array(opts.knobs || 1)].map(() => new Variable(0));
		const knobs = knobPositions.map((_var, i) => {
			const config = {
				absoluteMin: new Variable(opts.min || 0),
				absoluteMax: new Variable(opts.max || 100),
				min: knobPositions[i - 1],
				max: knobPositions[i + 1],
				axisLength: this.axisLength,
				axisHeight: this.axisHeight
			};
			return new Knob(this.sliderAxis, _var, config, changeCallback);
		});
		this.uiValues = knobs.map(knob => knob.uiValue);

		const props = {
			userSelect: 'none',
			style: {opacity: visibility},
			attached: this.setDimensions.bind(this, this.sliderAxis, visibility)
		};
		if (this.options.ready) props.ready = this.options.ready.bind(this);
		const displayElements = this.configDisplayElements();
		parent.appendChild(new Div('.sliderUI', displayElements, props));
		return knobs; // to allow Public API
	}

	configDisplayElements(){
		const children = [this.sliderAxis];
		if (this.options.addLabel == false) return children;

		// create reactive variable for UI display info, 0 - 100
		const uiValues = this.uiValues.reduce((arr, Var, i, values) => {
			const valueSeparator = this.options.valueSeparator || ' - '
			if (i > 0 && i < values.length) arr.push(valueSeparator);
			return arr.concat(Var.to(v => v.toFixed(2)));
		}, []);
		children.unshift(P([this.options.label, ' ', Span([uiValues])]));
		return children;
	}

	setDimensions(el, visibility){
		const dimentions = getElementSize(el);
		this.axisLength.put(dimentions.width);
		this.axisHeight.put(dimentions.height);
		if (this.options.attached) this.options.attached.call(this);
		setTimeout(() => visibility.put(1), 1); // component has mounted
	}
}


module.exports = Slider;
